﻿--[[
Name: Rating Buster
Revision: $Revision: 2 $
Developed by: Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
Description: Converts combat ratings in tooltips into normal percentages.
]]

-----------------
-- Addon Setup --
-----------------

-- Version Info
local MAJOR_VERSION = "0.5.1"
--local MINOR_VERSION = tonumber((string.gsub("$Revision: 2 $", "^.-(%d+).-$", "%1")))
-- Librarys
local Compost = AceLibrary("Compost-2.0")
--local Dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("RatingBuster")
-- AceAddon Initialization
RatingBuster = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
RatingBuster.title = "Rating Buster"
RatingBuster.version = MAJOR_VERSION
--RatingBuster.date = string.gsub("$Date: 2006-11-05 15:10:31 -0700 (Sun, 03 Sep 2006) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")

-- Set Debugging
RatingBuster:SetDebugging(false)

---------------------
-- Saved Variables --
---------------------
-- Register DB
RatingBuster:RegisterDB("RatingBusterDB")
-- Default values
RatingBuster.DBDefaults = {
	showItemLevel = true,
	showItemID = false,
}
-- Register Defaults
RatingBuster:RegisterDefaults("profile", RatingBuster.DBDefaults)

---------------------------
-- Slash Command Options --
---------------------------

local consoleOptions = {
	type='group',
	args = {
		itemid = {
			type = 'toggle',
			name = L["Show Item ID"],
			desc = L["Toggle display of Item ID in tooltip"],
			get = function() return RatingBuster.db.profile.showItemID end,
			set = function(v) RatingBuster.db.profile.showItemID = v end,
		},
		itemlevel = {
			type = 'toggle',
			name = L["Show Item Level"],
			desc = L["Toggle display of Item Level in tooltip"],
			get = function() return RatingBuster.db.profile.showItemLevel end,
			set = function(v) RatingBuster.db.profile.showItemLevel = v end,
		},
		targetlevel = {
			type = 'range',
			name = L["Target Level"],
			desc = L["Changes the Target Level used in rating calculations"],
			get = function() return RatingBuster.targetLevel end,
			set = function(v) RatingBuster.targetLevel = v end,
			min = 10,
			max = 73, -- set to level cap + 3
			step = 1,
		},
	},
}

RatingBuster:RegisterChatCommand({"/rb", "/ratingbuster"}, consoleOptions)

---------------------
-- Initializations --
---------------------

function RatingBuster:OnInitialize()
	-- Build the RatingTypeToID table from RatingTypeCheckList table
	self:BuildRatingTypeToID()
	-- build handler functions
	self:BuildTooltipHandlers()
	-- Hooking moved from OnEnable to OnInitialize until AceHook is fixed, to prevent multi hooking when resuming from standby
	-- Hook tooltip SetX methods
	self:HookTooltipMethods()
end

function RatingBuster:BuildRatingTypeToID()
	RatingBuster.RatingTypeToID = {}
	for _, v in ipairs(RatingTypeCheckList) do
		self.RatingTypeToID[v[1]] = v[2]
	end
end

function RatingBuster:OnEnable()
	-- Initialize targetLevel
	RatingBuster.targetLevel = UnitLevel("player")
	-- Check player level for lowbies
	if RatingBuster.targetLevel < 10 then
		RatingBuster.targetLevel = 10
	end
	-- Event to monitor shift, alt, ctrl keys
	self:RegisterEvent("MODIFIER_STATE_CHANGED")
	-- for setting a new level
	self:RegisterEvent("PLAYER_LEVEL_UP")
end

function RatingBuster:OnDisable()
end

-- event = MODIFIER_STATE_CHANGED 
-- arg1 = SHIFT or CTRL or ALT 
-- arg2 = 1 for down, 0 for up 
function RatingBuster:MODIFIER_STATE_CHANGED()
	--self:Debug("** "..arg1.." = "..arg2.." **") -- debug
end

-- event = PLAYER_LEVEL_UP
-- arg1 = New player level
function RatingBuster:PLAYER_LEVEL_UP()
	RatingBuster.targetLevel = arg1
end


-------------------------------
-- Hook Tooltip SetX Methods --
-------------------------------
RatingBuster.TooltipList = {
	ItemRefTooltip,
	GameTooltip,
	ShoppingTooltip1,
	ShoppingTooltip2,
	--EquipCompare support
	ComparisonTooltip1,
	ComparisonTooltip2,
	-- socketing interface
	--ItemSocketingDescription, -- commented out cause can't get it to work
}

RatingBuster.MethodList = {
	["SetHyperlink"] = "a",
	["SetBagItem"] = "GetContainerItemLink(a, b)",
	["SetInventoryItem"] = "GetInventoryItemLink(a, b)",
	-- auction
	["SetAuctionItem"] = "GetAuctionItemLink(a, b)",
	["SetAuctionSellItem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- get tooltip line 1
	-- loot
	["SetLootItem"] = "GetLootSlotLink(a, b)",
	["SetLootRollItem"] = "GetLootRollItemLink(a, b)",
	-- crafting
	["SetCraftSpell"] = "GetCraftItemLink(a)",
	["SetCraftItem"] = "GetCraftReagentItemLink(a, b)",
	["SetTradeSkillItem"] = "nil;if b then link=GetTradeSkillReagentItemLink(a, b) else link=GetTradeSkillItemLink(a) end",
	["SetTrainerService"] = "GetTrainerServiceItemLink(a)",-- profession trainer
	-- mail
	["SetInboxItem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- get tooltip line 1
	["SetSendMailItem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- get tooltip line 1
	-- buffs
	["SetPlayerBuff"] = "nil",
	-- quest log
	["SetQuestItem"] = "GetQuestItemLink(a, b)",
	["SetQuestLogItem"] = "GetQuestLogItemLink(a, b)",
	-- trade
	["SetTradePlayerItem"] = "GetTradePlayerItemLink(a)",
	["SetTradeTargetItem"] = "GetTradeTargetItemLink(a)",
	-- vendor tooltip
	["SetMerchantItem"] = "GetMerchantItemLink(a)",
	["SetMerchantCompareItem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- get tooltip line 1
	["SetBuybackItem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- get tooltip line 1
	-- socketing interface
	--"SetSocketedItem", -- main display
	["SetSocketGem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- tooltips -- get tooltip line 1
	["SetExistingSocketGem"] = "getglobal(GameTooltip:GetName()..'TextLeft1'):GetText()", -- tooltips -- get tooltip line 1
}
--[[
function RatingBuster.SetHyperlinkHook(tooltip, a, b)
	local link = a
	RatingBuster.ProcessTooltip(tooltip, link)
end
--]]
-- build handler functions
function RatingBuster:BuildTooltipHandlers()
	for methodName, getLink in pairs(self.MethodList) do
		handlerString = "function RatingBuster."..methodName.."Hook(tooltip, a, b) local link = "..getLink.." RatingBuster.ProcessTooltip(tooltip, link) end"
		loadstring(handlerString)()
	end
end

function RatingBuster:HookTooltipMethods()
	for _, tooltip in ipairs(self.TooltipList) do
		if tooltip then
			for methodName, _ in pairs(self.MethodList) do
				hooksecurefunc(tooltip, methodName, RatingBuster[methodName.."Hook"])
				-- prints out an error when put on standby, as of Rev: 15813
				--self:SecureHook(tooltip, methodName, self.ProcessTooltip)
			end
		end
	end
end


--------------------------
-- Process Tooltip Core --
--------------------------
--[[ unfinished methods
RatingBuster.OriginalText = {
	[RB_WEAPON_SKILL] = {
		"Equip: Increases axe skill rating by 17.",
		"Equip: Increases Dagger skill rating by 17.",
		"Equip: Increases sword skill rating by 17.",
		"Equip: Increases Two-Handed Swords skill rating by 19.",
		"Equip: Increases Bow skill rating by 9.",
		"Equip: Increases Crossbow skill rating by 9.",
		"Equip: Increases gun skill rating by 9.",
	},
	[RB_DEFENSE_SKILL] = {
		"+%d Defense Rating",
		"Equip: Increases defense rating by %d.",--Guardian Blade
		"Increases your armor by 2000 and defense skill rating by 30 for $d. Every time you take melee or ranged damage, this bonus is reduced by 200 armor and 3 defense.",
	},
	[RB_DODGE] = {
	},
	[RB_PARRY] = {
		"Equip: Increases your parry rating by 20.",--Sang'thraze the Deflector
	},
	[RB_BLOCK] = {
		"Equip: Increases your block rating by 13.",
		"Equip: Increases your shield block rating by 19.", --Watchkeeper's Kite Shield
	},
	[RB_HIT_MELEE] = {
		"+4 Hit Rating and +4 Agility", --Glinting Noble Topaz
		"Equip: Increases your hit rating by 10.",
		"Equip: Improves hit rating by 37.",--Khorium Champion
	},
	[RB_HIT_RANGED] = {
	},
	[RB_HIT_SPELL] = {
		"Equip: Increases your spell hit rating by 8.",
	},
	[RB_CRIT_MELEE] = {
		"+%d Critical Strike Rating",
		"Equip: Increases your critical strike rating by %d.",
		"Equip: Improves critical strike rating by 27.",--Felfury Gauntlets
	},
	[RB_CRIT_RANGED] = {
	},
	[RB_CRIT_SPELL] = {
		"Equip: Increases your spell critical strike rating by 14.",
		"Equip: Improves spell critical strike rating by 26.", --Pattern: Girdle of Ruination
	},
	[RB_HASTE_MELEE] = {
		"Chance on hit: Increases your haste rating by 200 for 10 sec.",
	},
	[RB_HASTE_RANGED] = {
	},
	[RB_HASTE_SPELL] = {
	},
	[RB_RESILIENCE] = {
		"Equip: Improves your resilience rating by 17.",--Resolute Cape
	},
}
RatingBuster.ShortConvertedText = {
	[RB_WEAPON_SKILL] = {
	},
	[RB_DEFENSE_SKILL] = {
		"+%d Defense Rating (%.2f)",
		"Equip: +%d Defense Rating (%.2f)",
	},
	[RB_DODGE] = {
	},
	[RB_PARRY] = {
	},
	[RB_BLOCK] = {
	},
	[RB_HIT_MELEE] = {
	},
	[RB_HIT_RANGED] = {
	},
	[RB_HIT_SPELL] = {
	},
	[RB_CRIT_MELEE] = {
		"+%d Critical Strike Rating (%.2f%%)",
		"Equip: +%d Critical Strike Rating (%.2f%%)",
	},
	[RB_CRIT_RANGED] = {
	},
	[RB_CRIT_SPELL] = {
	},
	[RB_HASTE_MELEE] = {
	},
	[RB_HASTE_RANGED] = {
	},
	[RB_HASTE_SPELL] = {
	},
	[RB_RESILIENCE] = {
	},
}
RatingBuster.TextRatingType = {
	[1] = RB_DEFENSE_SKILL,
	[2] = RB_CRIT_MELEE,
	[3] = RB_DEFENSE_SKILL,
	[4] = RB_CRIT_MELEE,
}
RatingBuster.Patterns = {
	{"^%+", "^%+$d:Rating $RatingType Rating$", "^%+$d:Rating $RatingType Rating %(%+$Percentage%%)$"},
	{"^Equip: Increases your", "^Equip: Increases your $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Improves your", "^Equip: Improves your $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Increases", "^Equip: Increases $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Improves", "^Equip: Improves $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Chance on hit:", "^Chance on hit: Increases your $RatingType rating by $d:Rating for $time sec%.$", "^Chance on hit: %+$d:Rating $RatingType Rating for $time sec %(%+$Percentage%%%)$"},
}
RatingBuster.Patterns2 = {
	{"^%+", "^%+$d:Rating $RatingType Rating$", "^%+$d:Rating $RatingType Rating %(%+$Percentage%%)$"},
	{"^Equip: Increases your", "^Equip: Increases your $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Improves your", "^Equip: Improves your $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Increases", "^Equip: Increases $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Equip: Improves", "^Equip: Improves $RatingType rating by $d:Rating%.$", "^Equip: %+$d:Rating $RatingType Rating %(%+$Percentage%%%)$"},
	{"^Chance on hit:", "^Chance on hit: Increases your $RatingType rating by $d:Rating for $time sec%.$", "^Chance on hit: %+$d:Rating $RatingType Rating for $time sec %(%+$Percentage%%%)$"},
	{"rating", ""}
}
--]]
--[[
Method 1
"+15 Crit Rating"
-> "+15(1.20%) Crit Rating"
"Equip: Increases your hit rating by 10."
-> "Equip: Increases your hit rating by 10(1.20%)."
Method 2
"+15 Crit Rating"
-> "+15 Crit Rating (1.20%)"
"Equip: Increases your hit rating by 10."
-> "Equip: Increases your hit rating by 10. (1.20%)"
Method 3
"+15 Crit Rating"
-> "+15 Crit Rating (1.20%)"
"Equip: Increases your hit rating by 10."
-> "Equip: +10 Hit Rating (1.20%)"
Method 4
+10 Sta (100HP) 
Equip: Adds 15 critical hit rating (.85% ) 
Equip: Adds 12 resilience (-.58% crit, -1.16% crit damage) 
--]]
function RatingBuster.ProcessTooltip(tooltip, link)
	-- check if in standby mode
	if not RatingBuster:IsActive() then return end
	-- Rating code
	for i = 2, tooltip:NumLines() do
		local fontString = getglobal(tooltip:GetName().."TextLeft"..i)
		local text = fontString:GetText()
		if text then
			lowerText = string.lower(text)
			if string.find(lowerText, L["rating"]) then
				local found, type, number
				-- Pattern 1: "(%d+)( ([^%d]-) [Rr]ating)"
				-- L["method1pattern1"] = "(%d+)( ([^%d]-) [Rr]ating)"
				-- Ex: 10 Crit Rating
				found, _, number, _, type = string.find(lowerText, L["method1pattern1"])
				if found then
					-- determine rating type
					local id = RatingBuster.RatingTypeToID[type]
					if id then
						local reversed, converted
						-- calculate reverse value
						reversed = ReverseRating(tonumber(number), id, RatingBuster.targetLevel)
						--RatingBuster:Debug(reversed..", "..tonumber(number)..", "..id..", "..RatingBuster.targetLevel)
						-- build converted string
						-- check if rating is a skill type
						if id == 1 or id == 2 then
							converted = string.gsub(text, L["method1pattern1"], "%1".." %("..string.format("%.2f", reversed).."%)".."%2")
						-- check if rating is resilience
						elseif id == 15 then
							converted = string.gsub(text, L["method1pattern1"], "%1".." %(%-"..string.format("%.2f", reversed).."%%%)".."%2")
						else
							converted = string.gsub(text, L["method1pattern1"], "%1".." %("..string.format("%.2f", reversed).."%%%)".."%2")
						end
						-- SetText
						fontString:SetText(converted)
					else
						--debug
						RatingBuster:Debug("Unknown Type1: "..tostring(type))
					end
				end
				-- Pattern 2: " [Rr]ating by (%d+)"
				-- L["method1pattern2"] = " [Rr]ating by (%d+)"
				-- Ex: rating by 10
				-- I used a slower but more locale friendly algoritm
				-- First pass, find the rating number with "rating by (%d+)"
				-- Second pass, search lowerText for all RatingTypes until we find one
				-- then we gsub "rating by (%d+)" for "%0".."%("..string.format("%.2f", percentage).."%)"
				---
				-- TODO: Warlock Atiesh, ID:22589
				found, _, number = string.find(lowerText, L["method1pattern2"])
				if found then
					-- determine rating type
					local id, type
					for _, v in ipairs(RatingTypeCheckList) do
						if string.find(lowerText, v[1]..L["method1pattern2"]) then
							type = v[1]
							id = RatingBuster.RatingTypeToID[type]
							break
						end
					end
					if id then
						local reversed, converted
						-- calculate reverse value
						reversed = ReverseRating(tonumber(number), id, RatingBuster.targetLevel)
						-- build converted string
						-- check if rating is a skill type
						if id == 1 or id == 2 then
							converted = string.gsub(text, L["method1pattern2"], "%0".." %("..string.format("%.2f", reversed).."%)")
						-- check if rating is resilience
						elseif id == 15 then
							converted = string.gsub(text, L["method1pattern2"], "%0".." %(%-"..string.format("%.2f", reversed).."%%%)")
						else
							converted = string.gsub(text, L["method1pattern2"], "%0".." %("..string.format("%.2f", reversed).."%%%)")
						end
						-- SetText
						fontString:SetText(converted)
					else
						--debug
						RatingBuster:Debug("Unknown Type2: "..tostring(type))
					end
				end
			end
			--[[ unfinished
			for _, pattern in ipairs(RatingBuster.Patterns) do
				-- fast pre check
				if string.find(text, pattern[1]) then
					local captured = RatingBuster.WCapture(text, pattern[2])
					--RatingBuster:Debug(tostring(rating)..", "..text..", "..original)
					if captured then
						-- determine rating type
						local ratingID = RatingBuster.RatingTypeToID[captured.RatingType]
						-- calculate percentage
						captured.Percentage = ReverseRating(tonumber(captured.Rating), ratingID)
						-- build converted string
						local converted = RatingBuster.WFormat(pattern, captured)
						-- SetText
						fontString:SetText(string.format(RatingBuster.ShortConvertedText[key], rating, ReverseRating(tonumber(rating), RatingBuster.TextRatingType[key])))
						break
					end
				end
			end
			--]]
		end
	end
	-- Item Level and Item ID
	-- Check for ItemLevel addon, do nothing if found
	if not ItemLevel_AddInfo and (RatingBuster.db.profile.showItemLevel or RatingBuster.db.profile.showItemID)then
		local id, level
		-- Get the Item ID from the link string
		if link then
			_, link, _, level = GetItemInfo(link)
			if link then
				_, _, id = string.find(link, "item:(%d+)")
			end
			local newLine = ""
			if level and RatingBuster.db.profile.showItemLevel then
				newLine = newLine..L["ItemLevel: "]..level
			end
			if id and RatingBuster.db.profile.showItemID then
				if newLine ~= "" then
					newLine = newLine..", "
				end
				newLine = newLine..L["ItemID: "]..id
			end
			if newLine ~= "" then
				tooltip:AddLine(newLine)
			end
		end
	end
	tooltip:Show()
end
--[[
-- WhitetoothCapture was taken and modified from my private Whitetooth Raid Warning System addon
function RatingBuster.WCapture(s, pattern)
	if type(s) ~= "string" or type(pattern) ~= "string" then return end
	--------------------------------
	-- Build variablesOrder table --
	--------------------------------
	-- stores the order which the variables appear in pattern
	local variablesOrder = Compost:Acquire()
	for variableName in string.gmatch(combatLogText, "%$([:%w]+)") do
		tinsert(variablesOrder, variableName)
	end
	-------------------
	-- Build pattern --
	-------------------
	-- transform pattern into string.find format
	-- replace $d:vars with (%d*) matches 0 or more, longest possible
	combatLogText = string.gsub(combatLogText, "%$[dD]:%w+", "(%%d*)")
	-- replace $vars with (.-) matches 0 or more, shortest possible
	combatLogText = string.gsub(combatLogText, "%$[%w]+", "(.-)")
	-----------------
	-- string.find --
	-----------------
	-- captures table, temp table for storing results from string.find
	local captures = Compost:Acquire(string.find(arg1, combatLogText))
	---------------------
	-- See what we got --
	---------------------
	-- if captures[1] == nil then nothing is found
	if not captures[1] then return end
	-- if captures[3] == nil then we have a match but no captures, just return true
	if not captures[3] then
		return true
	end
	-- now we are sure that we have somthing
	-- build and return capturedVariables table with proper keys and values
	local i = 3
	local capturedVariables = Compost:Acquire()
	for _, variableName in ipairs(variablesOrder) do
		capturedVariables[variableName] = captures[i]
		i = i + 1
	end
	-- clean up
	Compost:Reclaim(variablesOrder)
	Compost:Reclaim(captures)
	return capturedVariables
end
--]]